/*
  This file contains docstrings for use in the Python bindings.
  Do not edit! They were automatically extracted by ../gendoc.sh.
 */

#if defined(__GNUG__)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-variable"
#endif

namespace regina::python::doc {


// Docstring regina::python::doc::ExampleSnapPea
static const char *ExampleSnapPea =
R"doc(This class offers routines for constructing various example SnapPea
triangulations. This is analagous to the (much richer) Example<3>
class.)doc";

namespace ExampleSnapPea_ {

// Docstring regina::python::doc::ExampleSnapPea_::figureEight
static const char *figureEight =
R"doc(Returns a new triangulation of the figure eight knot complement.

Returns:
    the figure eight knot complement.)doc";

// Docstring regina::python::doc::ExampleSnapPea_::gieseking
static const char *gieseking =
R"doc(Returns a new triangulation of the Gieseking manifold.

Returns:
    the Gieseking manifold.)doc";

// Docstring regina::python::doc::ExampleSnapPea_::trefoil
static const char *trefoil =
R"doc(Returns a new triangulation of the trefoil knot complement.

Returns:
    the trefoil knot complement.)doc";

// Docstring regina::python::doc::ExampleSnapPea_::whiteheadLink
static const char *whiteheadLink =
R"doc(Returns a new triangulation of the Whitehead link complement.

Returns:
    the Whitehead link complement.)doc";

// Docstring regina::python::doc::ExampleSnapPea_::x101
static const char *x101 =
R"doc(Returns a new triangulation of the SnapPea census manifold ``x101``.

Returns:
    the SnapPea census manifold ``x101``.)doc";

}

} // namespace regina::python::doc

#if defined(__GNUG__)
#pragma GCC diagnostic pop
#endif

