/**
 *  @file end_namespace.h
 *  @brief Close a C++ namespace.
 *
 *  If you declared a namespace block in kernel_namespace.h then
 *  you can close the block here.
 */

} // namespace regina::snappea
